WEBAPP_URL = "https://jvn.io"
API_URL = "https://api.jovian.ai"
RC_FILENAME = ".jovianrc"
FILENAME_MSG = 'Failed to detect notebook filename. Please provide the notebook filename (including .ipynb extension) as the "filename" argument to "jovian.commit".'
